@@ -13,14 +13,14 @@
 # limitations under the License.
 #
 # SPDX-License-Identifier: Apache-2.0
-#
-
+# Copyright (c) OWASP Foundation. All Rights Reserved.
 import hashlib
 import re
 import sys
 import warnings
+from datetime import datetime
 from enum import Enum
-from typing import List, Optional, Union
+from typing import Iterable, Optional, Set
 
 from ..exception.model import InvalidLocaleTypeException, InvalidUriException, NoPropertiesProvidedException, \
     MutuallyExclusivePropertiesException, UnknownHashTypeException
@@ -51,9 +51,84 @@ def sha1sum(filename: str) -> str:
     return h.hexdigest()
 
 
+class DataFlow(Enum):
+    """
+    This is our internal representation of the dataFlowType simple type within the CycloneDX standard.
+
+    .. note::
+        See the CycloneDX Schema: https://cyclonedx.org/docs/1.4/xml/#type_dataFlowType
+    """
+    INBOUND = "inbound"
+    OUTBOUND = "outbound"
+    BI_DIRECTIONAL = "bi-directional"
+    UNKNOWN = "unknown"
+
+
+class DataClassification:
+    """
+    This is our internal representation of the `dataClassificationType` complex type within the CycloneDX standard.
+
+    .. note::
+        See the CycloneDX Schema for dataClassificationType:
+        https://cyclonedx.org/docs/1.4/xml/#type_dataClassificationType
+    """
+
+    def __init__(self, *, flow: DataFlow, classification: str) -> None:
+        self.flow = flow
+        self.classification = classification
+
+    @property
+    def flow(self) -> DataFlow:
+        """
+        Specifies the flow direction of the data.
+
+        Valid values are: inbound, outbound, bi-directional, and unknown.
+
+        Direction is relative to the service.
+
+        - Inbound flow states that data enters the service
+        - Outbound flow states that data leaves the service
+        - Bi-directional states that data flows both ways
+        - Unknown states that the direction is not known
+
+        Returns:
+            `DataFlow`
+        """
+        return self._flow
+
+    @flow.setter
+    def flow(self, flow: DataFlow) -> None:
+        self._flow = flow
+
+    @property
+    def classification(self) -> str:
+        """
+        Data classification tags data according to its type, sensitivity, and value if altered, stolen, or destroyed.
+
+        Returns:
+            `str`
+        """
+        return self._classification
+
+    @classification.setter
+    def classification(self, classification: str) -> None:
+        self._classification = classification
+
+    def __eq__(self, other: object) -> bool:
+        if isinstance(other, DataClassification):
+            return hash(other) == hash(self)
+        return False
+
+    def __hash__(self) -> int:
+        return hash((self.flow, self.classification))
+
+    def __repr__(self) -> str:
+        return f'<DataClassification flow={self.flow}>'
+
+
 class Encoding(Enum):
     """
-    This is out internal representation of the encoding simple type within the CycloneDX standard.
+    This is our internal representation of the encoding simple type within the CycloneDX standard.
 
     .. note::
         See the CycloneDX Schema: https://cyclonedx.org/docs/1.4/#type_encoding
@@ -71,7 +146,7 @@ class AttachedText:
 
     DEFAULT_CONTENT_TYPE = 'text/plain'
 
-    def __init__(self, content: str, content_type: str = DEFAULT_CONTENT_TYPE,
+    def __init__(self, *, content: str, content_type: str = DEFAULT_CONTENT_TYPE,
                  encoding: Optional[Encoding] = None) -> None:
         self.content_type = content_type
         self.encoding = encoding
@@ -122,6 +197,17 @@ class AttachedText:
     def content(self, content: str) -> None:
         self._content = content
 
+    def __eq__(self, other: object) -> bool:
+        if isinstance(other, AttachedText):
+            return hash(other) == hash(self)
+        return False
+
+    def __hash__(self) -> int:
+        return hash((self.content, self.content_type, self.encoding))
+
+    def __repr__(self) -> str:
+        return f'<AttachedText content-type={self.content_type}, encoding={self.encoding}>'
+
 
 class HashAlgorithm(Enum):
     """
@@ -191,18 +277,48 @@ class HashType:
 
         raise UnknownHashTypeException(f"Unable to determine hash type from '{composite_hash}'")
 
-    def __init__(self, algorithm: HashAlgorithm, hash_value: str) -> None:
-        self._alg = algorithm
-        self._content = hash_value
+    def __init__(self, *, algorithm: HashAlgorithm, hash_value: str) -> None:
+        self.alg = algorithm
+        self.content = hash_value
 
-    def get_algorithm(self) -> HashAlgorithm:
+    @property
+    def alg(self) -> HashAlgorithm:
+        """
+        Specifies the algorithm used to create the hash.
+
+        Returns:
+            `HashAlgorithm`
+        """
         return self._alg
 
-    def get_hash_value(self) -> str:
+    @alg.setter
+    def alg(self, alg: HashAlgorithm) -> None:
+        self._alg = alg
+
+    @property
+    def content(self) -> str:
+        """
+        Hash value content.
+
+        Returns:
+            `str`
+        """
         return self._content
 
+    @content.setter
+    def content(self, content: str) -> None:
+        self._content = content
+
+    def __eq__(self, other: object) -> bool:
+        if isinstance(other, HashType):
+            return hash(other) == hash(self)
+        return False
+
+    def __hash__(self) -> int:
+        return hash((self.alg, self.content))
+
     def __repr__(self) -> str:
-        return f'<Hash {self._alg.value}:{self._content}>'
+        return f'<HashType {self.alg.name}:{self.content}>'
 
 
 class ExternalReferenceType(Enum):
@@ -253,10 +369,16 @@ class XsUri:
 
     def __eq__(self, other: object) -> bool:
         if isinstance(other, XsUri):
-            return str(self) == str(other)
+            return hash(other) == hash(self)
         return False
 
+    def __hash__(self) -> int:
+        return hash(self._uri)
+
     def __repr__(self) -> str:
+        return f'<XsUri {self._uri}>'
+
+    def __str__(self) -> str:
         return self._uri
 
 
@@ -269,61 +391,85 @@ class ExternalReference:
         See the CycloneDX Schema definition: https://cyclonedx.org/docs/1.3/#type_externalReference
     """
 
-    def __init__(self, reference_type: ExternalReferenceType, url: Union[str, XsUri], comment: str = '',
-                 hashes: Optional[List[HashType]] = None) -> None:
-        self._type: ExternalReferenceType = reference_type
-        self._url = str(url)
-        self._comment = comment
-        self._hashes: List[HashType] = hashes if hashes else []
+    def __init__(self, *, reference_type: ExternalReferenceType, url: XsUri, comment: Optional[str] = None,
+                 hashes: Optional[Iterable[HashType]] = None) -> None:
+        self.url = url
+        self.comment = comment
+        self.type = reference_type
+        self.hashes = set(hashes or [])
 
-    def add_hash(self, our_hash: HashType) -> None:
+    @property
+    def url(self) -> XsUri:
         """
-        Adds a hash that pins/identifies this External Reference.
-
-        Args:
-            our_hash:
-                `HashType` instance
-        """
-        self._hashes.append(our_hash)
-
-    def get_comment(self) -> Union[str, None]:
-        """
-        Get the comment for this External Reference.
+        The URL to the external reference.
 
         Returns:
-            Any comment as a `str` else `None`.
-        """
-        return self._comment
-
-    def get_hashes(self) -> List[HashType]:
-        """
-        List of cryptographic hashes that identify this External Reference.
-
-        Returns:
-            `List` of `HashType` objects where there are any hashes, else an empty `List`.
-        """
-        return self._hashes
-
-    def get_reference_type(self) -> ExternalReferenceType:
-        """
-        Get the type of this External Reference.
-
-        Returns:
-            `ExternalReferenceType` that represents the type of this External Reference.
-        """
-        return self._type
-
-    def get_url(self) -> str:
-        """
-        Get the URL/URI for this External Reference.
-
-        Returns:
-            URI as a `str`.
+            `XsUri`
         """
         return self._url
 
+    @url.setter
+    def url(self, url: XsUri) -> None:
+        self._url = url
+
+    @property
+    def comment(self) -> Optional[str]:
+        """
+        An optional comment describing the external reference.
+
+        Returns:
+            `str` if set else `None`
+        """
+        return self._comment
+
+    @comment.setter
+    def comment(self, comment: Optional[str]) -> None:
+        self._comment = comment
+
+    @property
+    def type(self) -> ExternalReferenceType:
+        """
+        Specifies the type of external reference.
+
+        There are built-in types to describe common references. If a type does not exist for the reference being
+        referred to, use the "other" type.
+
+        Returns:
+            `ExternalReferenceType`
+        """
+        return self._type
+
+    @type.setter
+    def type(self, type_: ExternalReferenceType) -> None:
+        self._type = type_
+
+    @property
+    def hashes(self) -> Set[HashType]:
+        """
+        The hashes of the external reference (if applicable).
+
+        Returns:
+            Set of `HashType`
+        """
+        return self._hashes
+
+    @hashes.setter
+    def hashes(self, hashes: Iterable[HashType]) -> None:
+        self._hashes = set(hashes)
+
+    def __eq__(self, other: object) -> bool:
+        if isinstance(other, ExternalReference):
+            return hash(other) == hash(self)
+        return False
+
+    def __hash__(self) -> int:
+        return hash((
+            self._type, self._url, self._comment,
+            tuple(sorted(self._hashes, key=hash))
+        ))
+
     def __repr__(self) -> str:
-        return f'<ExternalReference {self._type.name}, {self._url}> {self._hashes}'
+        return f'<ExternalReference {self._type.name}, {self._url}>'
 
 
 class License:
@@ -335,17 +481,17 @@ class License:
         See the CycloneDX Schema definition: https://cyclonedx.org/docs/1.4/xml/#type_licenseType
     """
 
-    def __init__(self, spxd_license_id: Optional[str] = None, license_name: Optional[str] = None,
+    def __init__(self, *, spdx_license_id: Optional[str] = None, license_name: Optional[str] = None,
                  license_text: Optional[AttachedText] = None, license_url: Optional[XsUri] = None) -> None:
-        if not spxd_license_id and not license_name:
-            raise MutuallyExclusivePropertiesException('Either `spxd_license_id` or `license_name` MUST be supplied')
-        if spxd_license_id and license_name:
+        if not spdx_license_id and not license_name:
+            raise MutuallyExclusivePropertiesException('Either `spdx_license_id` or `license_name` MUST be supplied')
+        if spdx_license_id and license_name:
             warnings.warn(
-                'Both `spxd_license_id` and `license_name` have been supplied - `license_name` will be ignored!',
+                'Both `spdx_license_id` and `license_name` have been supplied - `license_name` will be ignored!',
                 RuntimeWarning
             )
-        self.id = spxd_license_id
-        if not spxd_license_id:
+        self.id = spdx_license_id
+        if not spdx_license_id:
             self.name = license_name
         else:
             self.name = None
@@ -409,6 +555,17 @@ class License:
     def url(self, url: Optional[XsUri]) -> None:
         self._url = url
 
+    def __eq__(self, other: object) -> bool:
+        if isinstance(other, License):
+            return hash(other) == hash(self)
+        return False
+
+    def __hash__(self) -> int:
+        return hash((self.id, self.name, self.text, self.url))
+
+    def __repr__(self) -> str:
+        return f'<License id={self.id}, name={self.name}>'
+
 
 class LicenseChoice:
     """
@@ -419,18 +576,18 @@ class LicenseChoice:
         See the CycloneDX Schema definition: https://cyclonedx.org/docs/1.4/xml/#type_licenseChoiceType
     """
 
-    def __init__(self, license: Optional[License] = None, license_expression: Optional[str] = None) -> None:
-        if not license and not license_expression:
+    def __init__(self, *, license_: Optional[License] = None, license_expression: Optional[str] = None) -> None:
+        if not license_ and not license_expression:
             raise NoPropertiesProvidedException(
                 'One of `license` or `license_expression` must be supplied - neither supplied'
             )
-        if license and license_expression:
+        if license_ and license_expression:
             warnings.warn(
                 'Both `license` and `license_expression` have been supplied - `license` will take precedence',
                 RuntimeWarning
             )
-        self.license = license
-        if not license:
+        self.license = license_
+        if not license_:
             self.expression = license_expression
         else:
             self.expression = None
@@ -446,8 +603,8 @@ class LicenseChoice:
         return self._license
 
     @license.setter
-    def license(self, license: Optional[License]) -> None:
-        self._license = license
+    def license(self, license_: Optional[License]) -> None:
+        self._license = license_
 
     @property
     def expression(self) -> Optional[str]:
@@ -465,6 +622,17 @@ class LicenseChoice:
     def expression(self, expression: Optional[str]) -> None:
         self._expression = expression
 
+    def __eq__(self, other: object) -> bool:
+        if isinstance(other, LicenseChoice):
+            return hash(other) == hash(self)
+        return False
+
+    def __hash__(self) -> int:
+        return hash((self.license, self.expression))
+
+    def __repr__(self) -> str:
+        return f'<LicenseChoice license={self.license}, expression={self.expression}>'
+
 
 class Property:
     """
@@ -477,28 +645,51 @@ class Property:
     Specifies an individual property with a name and value.
     """
 
-    def __init__(self, name: str, value: str) -> None:
-        self._name = name
-        self._value = value
+    def __init__(self, *, name: str, value: str) -> None:
+        self.name = name
+        self.value = value
 
-    def get_name(self) -> str:
+    @property
+    def name(self) -> str:
         """
-        Get the name of this Property.
+        The name of the property.
+
+        Duplicate names are allowed, each potentially having a different value.
 
         Returns:
-            Name of this Property as `str`.
+            `str`
         """
         return self._name
 
-    def get_value(self) -> str:
+    @name.setter
+    def name(self, name: str) -> None:
+        self._name = name
+
+    @property
+    def value(self) -> str:
         """
-        Get the value of this Property.
+        Value of this Property.
 
         Returns:
-            Value of this Property as `str`.
+             `str`
         """
         return self._value
 
+    @value.setter
+    def value(self, value: str) -> None:
+        self._value = value
+
+    def __eq__(self, other: object) -> bool:
+        if isinstance(other, Property):
+            return hash(other) == hash(self)
+        return False
+
+    def __hash__(self) -> int:
+        return hash((self.name, self.value))
+
+    def __repr__(self) -> str:
+        return f'<Property name={self.name}>'
+
 
 class NoteText:
     """
@@ -511,7 +702,7 @@ class NoteText:
 
     DEFAULT_CONTENT_TYPE: str = 'text/plain'
 
-    def __init__(self, content: str, content_type: Optional[str] = None,
+    def __init__(self, *, content: str, content_type: Optional[str] = None,
                  content_encoding: Optional[Encoding] = None) -> None:
         self.content = content
         self.content_type = content_type or NoteText.DEFAULT_CONTENT_TYPE
@@ -561,6 +752,17 @@ class NoteText:
     def encoding(self, encoding: Optional[Encoding]) -> None:
         self._encoding = encoding
 
+    def __eq__(self, other: object) -> bool:
+        if isinstance(other, NoteText):
+            return hash(other) == hash(self)
+        return False
+
+    def __hash__(self) -> int:
+        return hash((self.content, self.content_type, self.encoding))
+
+    def __repr__(self) -> str:
+        return f'<NoteText content_type={self.content_type}, encoding={self.encoding}>'
+
 
 class Note:
     """
@@ -573,7 +775,7 @@ class Note:
 
     _LOCALE_TYPE_REGEX = re.compile(r'^[a-z]{2}(?:\-[A-Z]{2})?$')
 
-    def __init__(self, text: NoteText, locale: Optional[str] = None) -> None:
+    def __init__(self, *, text: NoteText, locale: Optional[str] = None) -> None:
         self.text = text
         self.locale = locale
 
@@ -617,6 +819,17 @@ class Note:
                     f"ISO-3166 (or higher) country code. according to ISO-639 format. Examples include: 'en', 'en-US'."
                 )
 
+    def __eq__(self, other: object) -> bool:
+        if isinstance(other, Note):
+            return hash(other) == hash(self)
+        return False
+
+    def __hash__(self) -> int:
+        return hash((self.text, self.locale))
+
+    def __repr__(self) -> str:
+        return f'<Note id={id(self)}, locale={self.locale}>'
+
 
 class OrganizationalContact:
     """
@@ -627,14 +840,14 @@ class OrganizationalContact:
         See the CycloneDX Schema definition: https://cyclonedx.org/docs/1.4/xml/#type_organizationalContact
     """
 
-    def __init__(self, name: Optional[str] = None, phone: Optional[str] = None, email: Optional[str] = None) -> None:
+    def __init__(self, *, name: Optional[str] = None, phone: Optional[str] = None, email: Optional[str] = None) -> None:
         if not name and not phone and not email:
             raise NoPropertiesProvidedException(
                 'One of name, email or phone must be supplied for an OrganizationalContact - none supplied.'
             )
-        self._name: Optional[str] = name
-        self._email: Optional[str] = email
-        self._phone: Optional[str] = phone
+        self.name = name
+        self.email = email
+        self.phone = phone
 
     @property
     def name(self) -> Optional[str]:
@@ -646,6 +859,10 @@ class OrganizationalContact:
         """
         return self._name
 
+    @name.setter
+    def name(self, name: Optional[str]) -> None:
+        self._name = name
+
     @property
     def email(self) -> Optional[str]:
         """
@@ -656,6 +873,10 @@ class OrganizationalContact:
         """
         return self._email
 
+    @email.setter
+    def email(self, email: Optional[str]) -> None:
+        self._email = email
+
     @property
     def phone(self) -> Optional[str]:
         """
@@ -666,6 +887,21 @@ class OrganizationalContact:
         """
         return self._phone
 
+    @phone.setter
+    def phone(self, phone: Optional[str]) -> None:
+        self._phone = phone
+
+    def __eq__(self, other: object) -> bool:
+        if isinstance(other, OrganizationalContact):
+            return hash(other) == hash(self)
+        return False
+
+    def __hash__(self) -> int:
+        return hash((self.name, self.phone, self.email))
+
+    def __repr__(self) -> str:
+        return f'<OrganizationalContact name={self.name}>'
+
 
 class OrganizationalEntity:
     """
@@ -676,15 +912,15 @@ class OrganizationalEntity:
         See the CycloneDX Schema definition: https://cyclonedx.org/docs/1.4/xml/#type_organizationalEntity
     """
 
-    def __init__(self, name: Optional[str] = None, urls: Optional[List[XsUri]] = None,
-                 contacts: Optional[List[OrganizationalContact]] = None) -> None:
+    def __init__(self, *, name: Optional[str] = None, urls: Optional[Iterable[XsUri]] = None,
+                 contacts: Optional[Iterable[OrganizationalContact]] = None) -> None:
         if not name and not urls and not contacts:
             raise NoPropertiesProvidedException(
                 'One of name, urls or contacts must be supplied for an OrganizationalEntity - none supplied.'
             )
-        self._name: Optional[str] = name
-        self._url: Optional[List[XsUri]] = urls
-        self._contact: Optional[List[OrganizationalContact]] = contacts
+        self.name = name
+        self.url = set(urls or [])
+        self.contact = set(contacts or [])
 
     @property
     def name(self) -> Optional[str]:
@@ -696,26 +932,49 @@ class OrganizationalEntity:
         """
         return self._name
 
+    @name.setter
+    def name(self, name: Optional[str]) -> None:
+        self._name = name
+
     @property
-    def urls(self) -> Optional[List[XsUri]]:
+    def url(self) -> Set[XsUri]:
         """
         Get a list of URLs of the organization. Multiple URLs are allowed.
 
         Returns:
-            `List[XsUri]` if set else `None`
+            Set of `XsUri`
         """
         return self._url
 
+    @url.setter
+    def url(self, urls: Iterable[XsUri]) -> None:
+        self._url = set(urls)
+
     @property
-    def contacts(self) -> Optional[List[OrganizationalContact]]:
+    def contact(self) -> Set[OrganizationalContact]:
         """
         Get a list of contact person at the organization. Multiple contacts are allowed.
 
         Returns:
-            `List[OrganizationalContact]` if set else `None`
+            Set of `OrganizationalContact`
         """
         return self._contact
 
+    @contact.setter
+    def contact(self, contacts: Iterable[OrganizationalContact]) -> None:
+        self._contact = set(contacts)
+
+    def __eq__(self, other: object) -> bool:
+        if isinstance(other, OrganizationalEntity):
+            return hash(other) == hash(self)
+        return False
+
+    def __hash__(self) -> int:
+        return hash((self.name, tuple(self.url), tuple(self.contact)))
+
+    def __repr__(self) -> str:
+        return f'<OrganizationalEntity name={self.name}>'
+
 
 class Tool:
     """
@@ -727,88 +986,206 @@ class Tool:
         See the CycloneDX Schema for toolType: https://cyclonedx.org/docs/1.3/#type_toolType
     """
 
-    def __init__(self, vendor: Optional[str] = None, name: Optional[str] = None, version: Optional[str] = None,
-                 hashes: Optional[List[HashType]] = None,
-                 external_references: Optional[List[ExternalReference]] = None) -> None:
-        self._vendor = vendor
-        self._name = name
-        self._version = version
-        self._hashes: List[HashType] = hashes or []
-        self._external_references: List[ExternalReference] = external_references or []
+    def __init__(self, *, vendor: Optional[str] = None, name: Optional[str] = None, version: Optional[str] = None,
+                 hashes: Optional[Iterable[HashType]] = None,
+                 external_references: Optional[Iterable[ExternalReference]] = None) -> None:
+        self.vendor = vendor
+        self.name = name
+        self.version = version
+        self.hashes = set(hashes or [])
+        self.external_references = set(external_references or [])
 
-    def add_external_reference(self, reference: ExternalReference) -> None:
+    @property
+    def vendor(self) -> Optional[str]:
         """
-        Add an external reference to this Tool.
-
-        Args:
-            reference:
-                `ExternalReference` to add to this Tool.
+        The name of the vendor who created the tool.
 
         Returns:
-            None
-        """
-        self._external_references.append(reference)
-
-    def add_external_references(self, references: List[ExternalReference]) -> None:
-        """
-        Add a list of external reference to this Tool.
-
-        Args:
-            references:
-                List of `ExternalReference` to add to this Tool.
-
-        Returns:
-            None
-        """
-        self._external_references = self._external_references + references
-
-    def get_external_references(self) -> List[ExternalReference]:
-        """
-        List of External References that relate to this Tool.
-
-        Returns:
-            `List` of `ExternalReference` objects where there are, else an empty `List`.
-        """
-        return self._external_references
-
-    def get_hashes(self) -> List[HashType]:
-        """
-        List of cryptographic hashes that identify this version of this Tool.
-
-        Returns:
-            `List` of `HashType` objects where there are any hashes, else an empty `List`.
-        """
-        return self._hashes
-
-    def get_name(self) -> Optional[str]:
-        """
-        The name of this Tool.
-
-        Returns:
-            `str` representing the name of the Tool
-        """
-        return self._name
-
-    def get_vendor(self) -> Optional[str]:
-        """
-        The vendor of this Tool.
-
-        Returns:
-            `str` representing the vendor of the Tool
+            `str` if set else `None`
         """
         return self._vendor
 
-    def get_version(self) -> Optional[str]:
+    @vendor.setter
+    def vendor(self, vendor: Optional[str]) -> None:
+        self._vendor = vendor
+
+    @property
+    def name(self) -> Optional[str]:
         """
-        The version of this Tool.
+        The name of the tool.
 
         Returns:
-            `str` representing the version of the Tool
+             `str` if set else `None`
+        """
+        return self._name
+
+    @name.setter
+    def name(self, name: Optional[str]) -> None:
+        self._name = name
+
+    @property
+    def version(self) -> Optional[str]:
+        """
+        The version of the tool.
+
+        Returns:
+             `str` if set else `None`
         """
         return self._version
 
+    @version.setter
+    def version(self, version: Optional[str]) -> None:
+        self._version = version
+
+    @property
+    def hashes(self) -> Set[HashType]:
+        """
+        The hashes of the tool (if applicable).
+
+        Returns:
+            Set of `HashType`
+        """
+        return self._hashes
+
+    @hashes.setter
+    def hashes(self, hashes: Iterable[HashType]) -> None:
+        self._hashes = set(hashes)
+
+    @property
+    def external_references(self) -> Set[ExternalReference]:
+        """
+        External References provide a way to document systems, sites, and information that may be relevant but which
+        are not included with the BOM.
+
+        Returns:
+            Set of `ExternalReference`
+        """
+        return self._external_references
+
+    @external_references.setter
+    def external_references(self, external_references: Iterable[ExternalReference]) -> None:
+        self._external_references = set(external_references)
+
+    def __eq__(self, other: object) -> bool:
+        if isinstance(other, Tool):
+            return hash(other) == hash(self)
+        return False
+
+    def __hash__(self) -> int:
+        return hash((self.vendor, self.name, self.version, tuple(self.hashes), tuple(self.external_references)))
+
     def __repr__(self) -> str:
-        return '<Tool {}:{}:{}>'.format(self._vendor, self._name, self._version)
+        return f'<Tool name={self.name}, version={self.version}, vendor={self.vendor}>'
+
+
+class IdentifiableAction:
+    """
+    This is out internal representation of the `identifiableActionType` complex type.
+
+    .. note::
+        See the CycloneDX specification: https://cyclonedx.org/docs/1.4/xml/#type_identifiableActionType
+    """
+
+    def __init__(self, *, timestamp: Optional[datetime] = None, name: Optional[str] = None,
+                 email: Optional[str] = None) -> None:
+        if not timestamp and not name and not email:
+            raise NoPropertiesProvidedException(
+                'At least one of `timestamp`, `name` or `email` must be provided for an `IdentifiableAction`.'
+            )
+
+        self.timestamp = timestamp
+        self.name = name
+        self.email = email
+
+    @property
+    def timestamp(self) -> Optional[datetime]:
+        """
+        The timestamp in which the action occurred.
+
+        Returns:
+            `datetime` if set else `None`
+        """
+        return self._timestamp
+
+    @timestamp.setter
+    def timestamp(self, timestamp: Optional[datetime]) -> None:
+        self._timestamp = timestamp
+
+    @property
+    def name(self) -> Optional[str]:
+        """
+        The name of the individual who performed the action.
+
+        Returns:
+            `str` if set else `None`
+        """
+        return self._name
+
+    @name.setter
+    def name(self, name: Optional[str]) -> None:
+        self._name = name
+
+    @property
+    def email(self) -> Optional[str]:
+        """
+        The email address of the individual who performed the action.
+
+        Returns:
+            `str` if set else `None`
+        """
+        return self._email
+
+    @email.setter
+    def email(self, email: Optional[str]) -> None:
+        self._email = email
+
+    def __eq__(self, other: object) -> bool:
+        if isinstance(other, IdentifiableAction):
+            return hash(other) == hash(self)
+        return False
+
+    def __hash__(self) -> int:
+        return hash((self.timestamp, self.name, self.email))
+
+    def __repr__(self) -> str:
+        return f'<IdentifiableAction name={self.name}, email={self.email}>'
+
+
+class Copyright:
+    """
+    This is out internal representation of the `copyrightsType` complex type.
+
+    .. note::
+        See the CycloneDX specification: https://cyclonedx.org/docs/1.4/xml/#type_copyrightsType
+    """
+
+    def __init__(self, *, text: str) -> None:
+        self.text = text
+
+    @property
+    def text(self) -> str:
+        """
+        Copyright statement.
+
+        Returns:
+            `str` if set else `None`
+        """
+        return self._text
+
+    @text.setter
+    def text(self, text: str) -> None:
+        self._text = text
+
+    def __eq__(self, other: object) -> bool:
+        if isinstance(other, Copyright):
+            return hash(other) == hash(self)
+        return False
+
+    def __hash__(self) -> int:
+        return hash(self.text)
+
+    def __repr__(self) -> str:
+        return f'<Copyright text={self.text}>'
 
 
 if sys.version_info >= (3, 8):
@@ -821,7 +1198,7 @@ try:
 except Exception:
     __ThisToolVersion = None
 ThisTool = Tool(vendor='CycloneDX', name='cyclonedx-python-lib', version=__ThisToolVersion or 'UNKNOWN')
-ThisTool.add_external_references(references=[
+ThisTool.external_references.update([
     ExternalReference(
         reference_type=ExternalReferenceType.BUILD_SYSTEM,
         url=XsUri('https://github.com/CycloneDX/cyclonedx-python-lib/actions')
