@@ -16,13 +16,13 @@
 #
 # SPDX-License-Identifier: Apache-2.0
 # Copyright (c) OWASP Foundation. All Rights Reserved.
-
 from datetime import datetime, timezone
-from typing import List, Optional
+from typing import Iterable, Optional, Set
 from uuid import uuid4, UUID
 
-from . import ThisTool, Tool
+from . import ExternalReference, OrganizationalContact, OrganizationalEntity, LicenseChoice, Property, ThisTool, Tool
 from .component import Component
+from .service import Service
 from ..parser import BaseParser
 
 
@@ -31,42 +31,26 @@ class BomMetaData:
     This is our internal representation of the metadata complex type within the CycloneDX standard.
 
     .. note::
-        See the CycloneDX Schema for Bom metadata: https://cyclonedx.org/docs/1.3/#type_metadata
+        See the CycloneDX Schema for Bom metadata: https://cyclonedx.org/docs/1.4/#type_metadata
     """
 
-    def __init__(self, tools: Optional[List[Tool]] = None) -> None:
+    def __init__(self, *, tools: Optional[Iterable[Tool]] = None,
+                 authors: Optional[Iterable[OrganizationalContact]] = None, component: Optional[Component] = None,
+                 manufacture: Optional[OrganizationalEntity] = None,
+                 supplier: Optional[OrganizationalEntity] = None,
+                 licenses: Optional[Iterable[LicenseChoice]] = None,
+                 properties: Optional[Iterable[Property]] = None) -> None:
         self.timestamp = datetime.now(tz=timezone.utc)
-        self.tools = tools if tools else []
+        self.tools = set(tools or [])
+        self.authors = set(authors or [])
+        self.component = component
+        self.manufacture = manufacture
+        self.supplier = supplier
+        self.licenses = set(licenses or [])
+        self.properties = set(properties or [])
 
         if not self.tools:
-            self.add_tool(ThisTool)
-
-        self.component: Optional[Component] = None
-
-    @property
-    def tools(self) -> List[Tool]:
-        """
-        Tools used to create this BOM.
-
-        Returns:
-            `List` of `Tool` objects where there are any, else an empty `List`.
-        """
-        return self._tools
-
-    @tools.setter
-    def tools(self, tools: List[Tool]) -> None:
-        self._tools = tools
-
-    def add_tool(self, tool: Tool) -> None:
-        """
-        Add a Tool definition to this Bom Metadata. The `cyclonedx-python-lib` is automatically added - you do not need
-        to add this yourself.
-
-        Args:
-            tool:
-                Instance of `Tool` that represents the tool you are using.
-        """
-        self._tools.append(tool)
+            self.tools.add(ThisTool)
 
     @property
     def timestamp(self) -> datetime:
@@ -82,6 +66,38 @@ class BomMetaData:
     def timestamp(self, timestamp: datetime) -> None:
         self._timestamp = timestamp
 
+    @property
+    def tools(self) -> Set[Tool]:
+        """
+        Tools used to create this BOM.
+
+        Returns:
+            `Set` of `Tool` objects.
+        """
+        return self._tools
+
+    @tools.setter
+    def tools(self, tools: Iterable[Tool]) -> None:
+        self._tools = set(tools)
+
+    @property
+    def authors(self) -> Set[OrganizationalContact]:
+        """
+        The person(s) who created the BOM.
+
+        Authors are common in BOMs created through manual processes.
+
+        BOMs created through automated means may not have authors.
+
+        Returns:
+            Set of `OrganizationalContact`
+        """
+        return self._authors
+
+    @authors.setter
+    def authors(self, authors: Iterable[OrganizationalContact]) -> None:
+        self._authors = set(authors)
+
     @property
     def component(self) -> Optional[Component]:
         """
@@ -106,6 +122,81 @@ class BomMetaData:
         """
         self._component = component
 
+    @property
+    def manufacture(self) -> Optional[OrganizationalEntity]:
+        """
+        The organization that manufactured the component that the BOM describes.
+
+        Returns:
+            `OrganizationalEntity` if set else `None`
+        """
+        return self._manufacture
+
+    @manufacture.setter
+    def manufacture(self, manufacture: Optional[OrganizationalEntity]) -> None:
+        self._manufacture = manufacture
+
+    @property
+    def supplier(self) -> Optional[OrganizationalEntity]:
+        """
+        The organization that supplied the component that the BOM describes.
+
+        The supplier may often be the manufacturer, but may also be a distributor or repackager.
+
+        Returns:
+            `OrganizationalEntity` if set else `None`
+        """
+        return self._supplier
+
+    @supplier.setter
+    def supplier(self, supplier: Optional[OrganizationalEntity]) -> None:
+        self._supplier = supplier
+
+    @property
+    def licenses(self) -> Set[LicenseChoice]:
+        """
+        A optional list of statements about how this BOM is licensed.
+
+        Returns:
+            Set of `LicenseChoice`
+        """
+        return self._licenses
+
+    @licenses.setter
+    def licenses(self, licenses: Iterable[LicenseChoice]) -> None:
+        self._licenses = set(licenses)
+
+    @property
+    def properties(self) -> Set[Property]:
+        """
+        Provides the ability to document properties in a key/value store. This provides flexibility to include data not
+        officially supported in the standard without having to use additional namespaces or create extensions.
+
+        Property names of interest to the general public are encouraged to be registered in the CycloneDX Property
+        Taxonomy - https://github.com/CycloneDX/cyclonedx-property-taxonomy. Formal registration is OPTIONAL.
+
+        Return:
+            Set of `Property`
+        """
+        return self._properties
+
+    @properties.setter
+    def properties(self, properties: Iterable[Property]) -> None:
+        self._properties = set(properties)
+
+    def __eq__(self, other: object) -> bool:
+        if isinstance(other, BomMetaData):
+            return hash(other) == hash(self)
+        return False
+
+    def __hash__(self) -> int:
+        return hash((
+            self.timestamp, self.tools, self.component
+        ))
+
+    def __repr__(self) -> str:
+        return f'<BomMetaData timestamp={self.timestamp.utcnow()}>'
+
 
 class Bom:
     """
@@ -130,10 +221,12 @@ class Bom:
             `cyclonedx.model.bom.Bom`: A Bom instance that represents the valid data held in the supplied parser.
         """
         bom = Bom()
-        bom.add_components(parser.get_components())
+        bom.components.update(parser.get_components())
         return bom
 
-    def __init__(self) -> None:
+    def __init__(self, *, components: Optional[Iterable[Component]] = None,
+                 services: Optional[Iterable[Service]] = None,
+                 external_references: Optional[Iterable[ExternalReference]] = None) -> None:
         """
         Create a new Bom that you can manually/programmatically add data to later.
 
@@ -142,7 +235,9 @@ class Bom:
         """
         self.uuid = uuid4()
         self.metadata = BomMetaData()
-        self._components: List[Component] = []
+        self.components = set(components or [])
+        self.services = set(services or [])
+        self.external_references = set(external_references or [])
 
     @property
     def uuid(self) -> UUID:
@@ -176,58 +271,22 @@ class Bom:
         self._metadata = metadata
 
     @property
-    def components(self) -> List[Component]:
+    def components(self) -> Set[Component]:
         """
         Get all the Components currently in this Bom.
 
         Returns:
-             List of all Components in this Bom.
+             Set of `Component` in this Bom
         """
         return self._components
 
     @components.setter
-    def components(self, components: List[Component]) -> None:
-        self._components = components
-
-    def add_component(self, component: Component) -> None:
-        """
-        Add a Component to this Bom instance.
-
-        Args:
-            component:
-                `cyclonedx.model.component.Component` instance to add to this Bom.
-
-        Returns:
-            None
-        """
-        if not self.has_component(component=component):
-            self._components.append(component)
-
-    def add_components(self, components: List[Component]) -> None:
-        """
-        Add multiple Components at once to this Bom instance.
-
-        Args:
-            components:
-                List of `cyclonedx.model.component.Component` instances to add to this Bom.
-
-        Returns:
-            None
-        """
-        self.components = self._components + components
-
-    def component_count(self) -> int:
-        """
-        Returns the current count of Components within this Bom.
-
-        Returns:
-             The number of Components in this Bom as `int`.
-        """
-        return len(self._components)
+    def components(self, components: Iterable[Component]) -> None:
+        self._components = set(components)
 
     def get_component_by_purl(self, purl: Optional[str]) -> Optional[Component]:
         """
-        Get a Component already in the Bom by it's PURL
+        Get a Component already in the Bom by its PURL
 
         Args:
              purl:
@@ -263,7 +322,35 @@ class Bom:
         Returns:
             `bool` - `True` if the supplied Component is part of this Bom, `False` otherwise.
         """
-        return component in self._components
+        return component in self.components
+
+    @property
+    def services(self) -> Set[Service]:
+        """
+        Get all the Services currently in this Bom.
+
+        Returns:
+             Set of `Service` in this BOM
+        """
+        return self._services
+
+    @services.setter
+    def services(self, services: Iterable[Service]) -> None:
+        self._services = set(services)
+
+    @property
+    def external_references(self) -> Set[ExternalReference]:
+        """
+        Provides the ability to document external references related to the BOM or to the project the BOM describes.
+
+        Returns:
+            Set of `ExternalReference`
+        """
+        return self._external_references
+
+    @external_references.setter
+    def external_references(self, external_references: Iterable[ExternalReference]) -> None:
+        self._external_references = set(external_references)
 
     def has_vulnerabilities(self) -> bool:
         """
@@ -273,8 +360,17 @@ class Bom:
             `bool` - `True` if at least one `cyclonedx.model.component.Component` has at least one Vulnerability,
                 `False` otherwise.
         """
-        for c in self.components:
-            if c.has_vulnerabilities():
-                return True
+        return any(c.has_vulnerabilities() for c in self.components)
 
+    def __eq__(self, other: object) -> bool:
+        if isinstance(other, Bom):
+            return hash(other) == hash(self)
         return False
+
+    def __hash__(self) -> int:
+        return hash((
+            self.uuid, self.metadata, tuple(self.components), tuple(self.services), tuple(self.external_references)
+        ))
+
+    def __repr__(self) -> str:
+        return f'<Bom uuid={self.uuid}>'
